<?php
/**
 * Paginator
 * @package     Me
 * @copyright   Copyright (C) 2012 MicroEngine Inc.
 */
class Me_Paginator
{
    /**
     * ページ位置
     * @var integer
     */
    private $position;
    /**
     * １ページあたりの最大アイテム数
     * @var integer
     */
    private $limit;
    /**
     * アイテム数
     * @var integer
     */
    private $count;
    /**
     * ページ位置パラメータ手前までのURL
     * @var string
     */
    private $url;
    /**
     * アイテム合計
     * @var boolean
     */
    public $total;
    /**
     * 前ページの有無
     * @var boolean
     */
    public $prev;
    /**
     * 次ページの有無
     * @var boolean
     */
    public $next;

    /**
     * コンストラクタ
     * @param array $rows
     * @param integer $position
     * @param integer $limit
     * @param integer $count
     * @param string $url 
     */
    public function __construct(&$rows, $position, $limit, $count, $url, $total)
    {
        $this->position = $position;
        $this->limit = $limit;
        $this->count = $count;
        $this->url = $url;
        $this->total = $total;
        
        $this->prev = ($position > 0);
        if ($count > $limit) {
            $this->next = true;
            unset($rows[$count -1]);
            $this->count = count($rows);
        } else {
            $this->next = false;
        }
    }
    
    /**
     * 前ページのURLを返す
     * @return string 
     */
    public function prev_url()
    {
        return $this->url . ($this->position -1);
    }

    /**
     * 次ページのURLを返す
     * @return string 
     */
    public function next_url()
    {
        return $this->url . ($this->position +1);
    }

    /**
     * そのページの開始位置を返す
     * @return string 
     */
    public function start_item()
    {
        $num = $this->position * $this->limit;
        if ($num > 0 || $this->count > 0) {
            $num++;
        } else {
            $num = 0;
        }
        return $num;
    }

    /**
     * そのページの終了位置を返す
     * @return string 
     */
    public function end_item()
    {
        return ($this->position * $this->limit) + $this->count;
    }
}