<?php
/**
 * Metag
 * @package     Metag
 * @copyright   Copyright (C) 2011 MicroEngine Inc.
 */
class Metag
{
    /**
     * MEタグID（primary key）
     * DBフィールド
     * @var int
     */
    public $_id;
    /**
     * pageテーブルのprimary key
     * DBフィールド
     * @var int
     */
    public $page_id;
    /**
     * repeat id
     * DBフィールド
     * @var string
     */
    public $repeat_id;
    /**
     * repeat no
     * DBフィールド
     * @var int
     */
    public $repeat_no;
    /**
     * Metag ID
     * DBフィールド
     * @var string
     */
    public $id;
    /**
     * 入力値
     * DBフィールド
     * @var string
     */
    public $node_value;
    /**
     * URL
     * DBフィールド
     * @var string
     */
    public $url;
    /**
     * 属性
     * DBフィールド
     * @var string
     */
    public $attr;
    /**
     * タグタイプ
     * @var string
     */
    public $node_type;
    /**
     * ブロック要素かどうか
     * @var boolean
     */
    public $is_block = true;
    /**
     * 開きタグ
     * @var boolean
     */
    public $is_open_tag;
    /**
     * 子要素
     * @var array
     */
    public $child_nodes;
    /**
     * 親要素名
     * @var string
     */
    public $parent_node_name;
    /**
     * 開きタグ
     * @var string
     */
    public $open_tag;
    /**
     * 閉じタグ
     * @var string
     */
    public $close_tag;
    /**
     * 参照ID
     * @var string
     */
    public $ref_id;
    /**
     * 参照リピートID
     * @var string
     */
    public $ref_repeat_id;
    /**
     * 参照ページID
     * @var string
     */
    public $ref_page_id;
    /**
     * 参照リンク
     * @var string
     */
    public $ref_link;
    /**
     * 初期値
     * @var stirng
     */
    public $initial_value;
    /**
     * 選択テキスト
     * @var string
     */
    public $selected_text;
    /**
     * アンカー
     * @var string
     */
    public $anchor;
    /**
     * 配信方式
     * @var string static / dynamic
     */
    public $output_method;
    /**
     * 属性リスト
     * @var array
     */
    public $attr_list;
    /**
     * repeat回数
     * @var int
     */
    public $repeat_count;
    /**
     * エラーメッセージ
     * @var string
     */
    public $errors;
    /**
     * ページパス
     * @var string
     */
    public $page_path;

    /**
     * 初期値設定
     * @param array &$row
     * @param boolean $set_initial_flag 初期値をセットするか、空文字をセットするか
     * @param boolean $set_child_flag MEタグの子ノードをvalueにセットするかどうか
     * @return boolean $update_flag データ更新フラグ
     */
    public function set_initial_value(&$row, $set_initial_flag = true, $set_child_flag = false)
    {
        $update_flag = true;
        if (!isset($row['node_value'])) {
            if ($set_initial_flag === false) {
                $row['node_value'] = '';
            } else if (isset($this->initial_value)) {
                // initial_valueがあればそれを使う。
                $row['node_value'] = $this->initial_value;
            } else if ($set_child_flag && $this->node_type !== 'date' 
                && $this->node_type !== 'img' && $this->node_type !== 'attr' && $this->node_type !== 'file'
                && $this->node_type !== 'list') {
                $child = $this->get_child_node();
                $row['node_value'] = $child->node_value;
            } else {
                $update_flag = false;
            }
        }
        return $update_flag;
    }

    /**
     * tag生成機能の初期値設定
     * @param string $text
     */
    public function set_initial_attr_value($text = null)
    {
        $this->selected_text = $text;
    }

    /**
     * 編集viewの出力
     * @param array $row
     */
    public function edit_view($row)
    {

    }

    /**
     * 出力
     */
    public function output()
    {
        return '';
    }

    /**
     * 子ノード取得
     * @param int $index_no
     * @return mixed
     */
    public function get_child_node($index_no = 0)
    {
        if ($index_no === null) {
            return $this->child_nodes;
        }
        if (isset($this->child_nodes[$index_no])) {
            return $this->child_nodes[$index_no];
        }
        return null;
    }

    /**
     * 編集ボックスのstyle属性文字列を返す
     * @param integer $repeat_no
     * @return string
     */
    protected function get_style($repeat_no = null)
    {
        $border_color = $this->get_border_color($repeat_no);

        $str = ' style="cursor: pointer; '
            . 'background: none repeat scroll 0% 0% transparent ! important; font: inherit ! important;'
            . 'text-transform: inherit ! important; text-decoration: inherit ! important;'
            . 'text-align: inherit ! important; text-indent: inherit ! important;'
            . 'letter-spacing: inherit ! important; color: inherit ! important; float: inherit ! important;'
            . 'margin: 0px ! important; padding: 0px ! important; overflow: inherit ! important;'
            . 'position: static ! important; vertical-align: inherit ! important; visibility: inherit ! important;'
            . 'display: inline ! important; ';
        if (Me_Util::is_ie()) {
            $str .= 'border: 1px solid ' . $border_color . ' ! important;';
        } else {
            $str .= 'outline: 1px solid ' . $border_color . ' ! important; border: 0px none ! important;';
        }
        $str .= '" ';
        return $str;
    }

    /**
     * 編集アイコンのstyle属性文字列を返す
     * @param integer $repeat_no
     * @return string
     */
    protected function get_icon_style($repeat_no = null)
    {
        $border_color = $this->get_border_color($repeat_no);

        $str = ' style="cursor: pointer; '
            . 'background-color: #FFFFFF ! important; margin: 0px ! important; padding: 0px ! important;'
            . 'border: 1px solid ' . $border_color . ' ! important;" ';
        return $str;
    }

    /**
     * border色取得
     * @param integer $repeat_no
     * @return string $border_color
     */
    protected function get_border_color($repeat_no = null)
    {
        // border色
        $border_color = 'red';
        if ($repeat_no === null) {
            $border_color = Me_Config::get_page('edit_border_color');
        } else {
            $border_color_settings = Me_Config::get_array('edit_border_color_repeat', 'page');
            if (count($border_color_settings) > 0) {
                $index = ($repeat_no - 1) % count($border_color_settings);
                $border_color = $border_color_settings[$index];
            }
        }
        return $border_color;
    }

    /**
     * open関数を返す
     * @param array &$row
     * @return string
     */
    protected function get_open_method(&$row)
    {
        $ret = "window.parent.me.page.open("
            . "'{$this->id}', this, '{$this->node_type}', '{$this->repeat_id}','{$row['repeat_no']}'"
            . ');return false;';
        return $ret;
    }

    /**
     * aタグ属性を返す
     * @param array $row
     * @return array $attr
     */
    protected function get_a_attr($row)
    {
        $attr = array();
        if (!empty($row['url'])) {
            $child_node = $this->get_child_node();

            if ($child_node !== null && isset($child_node->a_attr)) {
                $attr = $child_node->a_attr;
            }
            $attr['href'] = Me_Util::html_escape($row['url']);
        }
        return $attr;
    }

    /**
     * リンク機能（アンカー機能）
     * @param array $row
     * @param array $exclude_attr_list
     * @return array
     */
    protected function get_a($row, $exclude_attr_list = array())
    {
        $a_attr = $this->get_a_attr($row);
        $a_open = '';
        $a_close = '';

        if (!empty($row['url']) || !empty($row['attr']['anchor'])) {
            if (isset($exclude_attr_list['href'])) {
                // 除外属性にhrefが指定されていれば、hrefはそのままにする。
            } else if (!empty($row['url'])) {
                // 相対パスに変換する
                $a_attr['href'] = Me_Util::get_relative_path(SITE_PATH . $this->page_path, $row['url']);
            } else {
                unset($a_attr['href']);
            }
            if (isset($exclude_attr_list['target'])) {
                // 除外属性にtargetが指定されていれば、targetはそのままにする。
            } else if (!empty($row['attr']['target'])) {
                $a_attr['target'] = $row['attr']['target'];
            } else {
                unset($a_attr['attr']['target']);
            }
            if (!empty($row['attr']['anchor'])) {
                if (!isset($exclude_attr_list['id'])) {
                    $a_attr['id'] = $row['attr']['anchor'];
                }
                if (!isset($exclude_attr_list['name'])) {
                    $a_attr['name'] = $row['attr']['anchor'];
                }
            }

            $a_open .= '<a';
            foreach ($a_attr as $key => $val) {
                if (strlen($val) < 1) {
                    continue;
                }
                if (substr($key, 0, 1) === 'on') {
                    // onで始まる属性はエスケープせずに出力する。
                    $a_open .= ' ' . $key . '="' . $val . '"';
                } else {
                    $a_open .= ' ' . $key . '="' . Me_Util::html_escape($val) . '"';
                }
            }
            $a_open .= '>';
            $a_close = '</a>';
        }
        return array($a_open, $a_close);
    }

    /**
     * 自動アンカー機能
     * @param array $row
     * @return string
     */
    protected function get_anchor($row)
    {
        $id = $this->id;
        if ($this->anchor === 'on') {
            if (strlen($row['repeat_no']) > 0) {
                $id .= '-' . $row['repeat_no'];
            }
            return '<a id="' . $id . '" name="' . $id . '"></a>';
        }
        return '';
    }

    /**
     * URLをセット
     * @param array &$row
     */
    protected function set_url(&$row)
    {
        if ($this->ref_link === 'on') {
            $row['url'] = $row['ref_link_path'];
        } else if ($this->ref_link === 'anchor') {
            $id = (empty($this->ref_id)) ? $this->id : $this->ref_id;
            if (strlen($row['repeat_no']) > 0) {
                $id .= '-' . $row['repeat_no'];
            }
            if (SITE_PATH . $this->page_path === $row['ref_link_path']) {
                $row['url'] = '#' . $id;
            } else {
                $row['url'] = $row['ref_link_path'] . '#' . $id;
            }
        }
    }

    /**
     * タグ生成
     * @param Template_Form $form
     */
    public function generate_tag(Template_Form $form)
    {
        // open tag
        $open_tag = '<!--{' . $this->node_type;
        foreach ($this->attr_list as $attr) {
            $attr_value = $this->$attr;
            if ($attr === 'style') {
                $attr_value = preg_replace('/[\r\n]/', '', $attr_value);
            }
            if (strlen($attr_value) > 0) {
                $open_tag .= ' ' . $attr . '="' . Me_Util::html_escape($attr_value) . '"';
            }
        }
        $open_tag .= '}-->';
        $this->open_tag = $open_tag;

        // close tag
        $this->close_tag = '<!--{/' . $this->node_type . '}-->';
    }

    /**
     * 値の有無を確認
     * @param array $row
     * @return boolean
     */
    public function exist_node_value($row)
    {
        if (!empty($row) && strlen($row['node_value']) > 0) {
            return true;
        }
        return false;
    }
}