<?php
/**
 * Page Draft Dao
 * @package     Page
 * @copyright   Copyright (C) 2012 MicroEngine Inc.
 */
class Page_Draft_Dao extends Me_Dao
{
    /**
     * テーブル名
     * @var string
     */
    public $table_name = 'page_draft';
    /**
     * Entityクラス名
     * @var stirng
     */
    public $class_name = 'Page_Draft';
    /**
     * カラム定義配列
     * @var array
     */
    public $column_list = array(
        '_id',
        'page_id',
        'file_name',
        'title',
        'keywords',
        'description',
        'note',
        'comment',
        'user_id',
        'user_name',
        'create_date',
    );

    /**
     * 下書き一覧取得（ユーザー限定）
     * @param integer $user_id
     * @param integer $limit
     * @return array $rows
     */
    public function select_list($user_id, $limit = -1)
    {
        $query = <<< QUERY
select d.page_id, d._id, p.page_path, d.title, d.user_name, d.create_date 
from page_draft as d
left outer join page as p on (d.page_id = p._id) 
where d.user_id = 1
order by d.create_date desc limit -1
QUERY;

        $sql = new Me_Sql();
        $sql->set_query($query);
        $sql->bind('where', 3, $user_id);
        $sql->bind('order', 5, $limit);
        $rows = $sql->exec();
        return $rows;
    }
    
    /**
     * 下書き一覧取得（全て）
     * @param integer $limit
     * @return array $rows
     */
    public function select_all_list($limit = -1)
    {        
        $query = <<< QUERY
select d.page_id, d._id, p.page_path, d.title, d.user_name, d.create_date 
from page_draft as d
left outer join page as p on (d.page_id = p._id)
order by d.create_date desc limit -1
QUERY;

        $sql = new Me_Sql();
        $sql->set_query($query);
        $sql->bind('order', 5, $limit);
        $rows = $sql->exec();
        return $rows;        
    }
    
    /**
     * ページID指定でレコードを削除
     * @param string $page_id
     * @return boolean
     */
    public function delete_by_page_id($page_id)
    {
        if (strlen($page_id) <= 0) {
            return false;
        }

        $query = <<< QUERY
delete from page_draft
where page_id = 1
QUERY;

        $sql = new Me_Sql();
        $sql->set_query($query);
        $sql->bind('where', 3, $page_id);
        $sql->exec();
        return true;
    }

    /**
     * テンプレートID指定でレコードを削除
     * @param string $template_id
     * @return boolean
     */
    public function delete_by_template_id($template_id)
    {
        if (strlen($template_id) <= 0) {
            return false;
        }

        $query = <<< QUERY
delete from page_draft
where page_id in ( select _id from page where 
template_id = 1 )
QUERY;

        $sql = new Me_Sql();
        $sql->set_query($query);
        $sql->bind('template_id', 2, $template_id);
        $sql->exec();
        return true;
    }
}