<?php
/**
 * Page Form
 * @package     Page
 * @copyright   Copyright (C) 2011 MicroEngine Inc.
 */
class Page_Form extends Page
{
    /** Temporary Directory */
    const TEMPORARY_DIRECTORY = '/tmp';
    /** Files Directory */
    const FILES_DIRECTORY = '/files';

    /**
     * page内のmetagのデータ
     * @var array
     */
    public $data;
    /**
     *  削除予定ファイルリスト
     *  @var array
     */
    public $unlink_files;
    /**
     * テンプレートフォーム
     * @var Template_Form
     */
    public $template_form;
    /**
     * ドキュメント
     * @var Metag_Template
     */
    public $document;
    /**
     * 初期値セットフラグ
     * @var boolean
     */
    public $init = false;
    /**
     * 新ページパス
     * @var string
     */
    public $new_page_path;
    /**
     * 保存先（page=本稿、page_draft=下書き）
     * @var string
     */
    public $save;
    /**
     * 下書きID
     * @var string
     */
    public $draft_id;
    /**
     * 下書き編集
     * @var boolean
     */
    public $draft_edit = false;
    /**
     * リビジョンID
     * @var string
     */
    public $revision_id;
    /**
     * コメント
     * @var string
     */
    public $comment;
    /**
     * 戻る画面
     */
    public $back;
    /**
     * エラーフラグ
     * @var boolean
     */
    public $is_error;
    /**
     * エラーメッセージ
     * @var array
     */
    public $errors;

    /**
     * MEタグデータ取得
     * @param string $id
     * @param string $repeat_id
     * @param string $repeat_no
     * @param object $node
     * @return array 
     */
    public function get_row($id, $repeat_id = null, $repeat_no = null, $node = null)
    {
        $ret = null;
        if ($id === null) {
            // templateノードやhtmlノードの場合
        } else if (empty($repeat_id)) {
            if (isset($id) && isset($this->data[$id])) {
                $ret = $this->data[$id];
            } else if (isset($id)) {
                if ($node !== null && $node->node_type !== 'repeat') {
                    $ret = array(
                        'id' => $id,
                    );
                    $this->data[$id] = $ret;
                }
            }
        } else {
            // repeat
            if (isset($this->data[$repeat_id][$repeat_no][$id])) {
                $ret = $this->data[$repeat_id][$repeat_no][$id];
            } else {
                $ret = array(
                    'id' => $id,
                    'repeat_id' => $repeat_id,
                    'repeat_no' => $repeat_no,
                );
                $this->data[$repeat_id][$repeat_no][$id] = $ret;
            }
        }
        return $ret;
    }

    /**
     * MEタグデータ設定
     * attrにanchor以外を含む場合に$rowにセットしたうえで設定する。Imgタグなどで利用。
     * @param string $id
     * @param array $row
     * @param string $repeat_id
     * @param string $repeat_no
     */
    public function set_row($id, $row, $repeat_id = null, $repeat_no = null)
    {
        if (empty($repeat_id) || empty($repeat_no)) {
            $this->data[$id] = $row;
        } else {
            // repeat
            $this->data[$repeat_id][$repeat_no][$id] = $row;
        }
    }

    /**
     * MEタグ値設定
     * attrがanchorのみの簡易指定版
     * @param string $id
     * @param string $node_value
     * @param string $repeat_id
     * @param string $repeat_no
     * @param string $url
     * @param string $target
     * @param string $anchor
     */
    public function set_node_value($id, $node_value, $repeat_id = null, $repeat_no = null, $url = null, $target = null, $anchor = null)
    {
        if ($repeat_id === null || $repeat_no === null) {
            $this->data[$id]['node_value'] = $node_value;
            $this->data[$id]['url'] = $url;
            $this->data[$id]['attr']['target'] = $target;
            $this->data[$id]['attr']['anchor'] = $anchor;
        } else {
            // repeat
            $this->data[$repeat_id][$repeat_no][$id]['node_value'] = $node_value;
            $this->data[$repeat_id][$repeat_no][$id]['url'] = $url;
            $this->data[$repeat_id][$repeat_no][$id]['attr']['target'] = $target;
            $this->data[$repeat_id][$repeat_no][$id]['attr']['anchor'] = $anchor;
        }
    }

    /**
     * 削除予定ファイルセット
     * @param string $file_path
     */
    public function set_unlink_files($file_path)
    {
        if (empty($file_path)) {
            return;
        }
        if (!is_array($this->unlink_files)) {
            $this->unlink_files = array();
        }
        $this->unlink_files[$file_path] = $file_path;
    }

    /**
     * 削除予定ファイル解除
     * @param string $file_path
     */
    public function unset_unlink_files($file_path)
    {
        if (empty($file_path) || !is_array($this->unlink_files)) {
            return;
        }
        if (isset($this->unlink_files[$file_path])) {
            unset($this->unlink_files[$file_path]);
        }
    }

    /**
     * 削除予定ファイルを削除
     */
    public function exec_unlink_files()
    {
        if (empty($this->unlink_files)) {
            return;
        }
        foreach ($this->unlink_files as $page_path) {
            if (strlen($page_path) === 0) {
                continue;
            }
            $file_path = $default_file_path = DOCUMENT_ROOT . $page_path;
            // 一時ディレクトリ
            $tmp_dir = UPLOAD_ROOT . Page_Form::TEMPORARY_DIRECTORY . '/';
            // 対象ファイルのディレクトリ名
            $dir = dirname($file_path) . '/';

            // 一時ディレクトリ内のファイルのみ削除する
            if (strpos($dir, $tmp_dir) !== false) {
                // サムネイルを削除
                $thumbs_image_path = UPLOAD_ROOT . Metag_Logic::THUMBNAIL_DIRECTORY . '/' . basename($file_path);
                if (file_exists($thumbs_image_path) && is_writable($thumbs_image_path)) {
                    unlink($thumbs_image_path);
                }
                
                $i = 1;
                while (true) {
                    if ($i > 1) {
                        $file_path = preg_replace('/\.[^\.]*$/', '-' . $i . '$0', $default_file_path);
                    }
                    if (file_exists($file_path) && is_writable($file_path)) {
                        // ファイルを削除
                        if (!unlink($file_path)) {
                            break;
                        }

                        // fileタイプ対応
                        $files_dir = UPLOAD_ROOT . self::FILES_DIRECTORY . '/';

                        // 一時ディレクトリの場合
                        if (strpos($dir, $tmp_dir) !== false && strcmp($dir, $tmp_dir) > 0) {
                            // ディレクトリを削除。ただし、ファイルが残っている場合は削除しない。
                            if (count(scandir($dir)) <= 2 && !rmdir($dir)) {
                                break;
                            }
                            // filesディレクトリの場合
                        } else if (strpos($dir, $files_dir) !== false && strcmp($dir, $files_dir) > 0) {
                            // ディレクトリを削除。ただし、ファイルが残っている場合は削除しない。
                            if (count(scandir($dir)) <= 2 && !rmdir($dir)) {
                                break;
                            }
                        }
                    } else {
                        break;
                    }
                    $i++;
                }
            }
        }
    }
}