<?php
/**
 * Relation Dao
 * @package     Relation
 * @copyright   Copyright (C) 2011 MicroEngine Inc.
 */
class Relation_Dao extends Me_Dao
{
    /**
     * テーブル名
     * @var string
     */
    public $table_name = 'relation';
    /**
     * Entityクラス名
     * @var stirng
     */
    public $class_name = 'Relation';
    /**
     * カラム定義配列
     * @var array
     */
    public $column_list = array(
        '_id',
        'from_template',
        'ref_page',
        'ref_template',
    );

    /**
     * コンストラクタ
     */
    public function __construct()
    {

    }

    /**
     * 一括登録
     * @param array $relation_list
     * @param integer $from_template_id
     * @return boolean
     */
    public function register_array($relation_list, $from_template_id)
    {
        if (!is_array($relation_list) || !is_numeric($from_template_id)) {
            return false;
        }

        // 登録済みのレコードを一旦削除
        $query = <<< QUERY
delete from relation
where from_template = 0
QUERY;

        $sql = new Me_Sql();
        $sql->set_query($query);
        $sql->bind('where', 3, $from_template_id);
        $sql->exec();

        // レコード登録
        foreach ($relation_list as $relation) {
            $this->insert($relation);
        }
        return true;
    }

    /**
     * 関連ページ取得
     * @param string $page_id
     * @param string $template_id
     * @return array
     */
    public function select_relation_page_id($page_id, $template_id)
    {
        $query = <<< QUERY
select p._id
from relation as r
join page as p on (p.template_id = r.from_template)
where r.ref_page = 0
or r.ref_template = 0
order by r.from_template asc
QUERY;

        $sql = new Me_Sql();
        $sql->set_query($query);
        $sql->bind('where', 3, $page_id);
        $sql->bind('or', 3, $template_id);
        $rows = $sql->exec();

        return $rows;
    }

    /**
     * 参照元テンプレートID指定で削除する
     * @param string $template_id
     * @return boolean
     */
    public function delete_from_template($template_id)
    {
        if (empty($template_id)) {
            return false;
        }

        $query = <<< QUERY
delete from relation
where from_template = 0
QUERY;

        $sql = new Me_Sql();
        $sql->set_query($query);
        $sql->bind('where', 3, $template_id);
        $sql->exec();
        return true;
    }

    /**
     * 参照先ページID指定で削除する
     * @param string $page_id
     * @return boolean
     */
    public function delete_from_page($page_id)
    {
        if (empty($page_id)) {
            return false;
        }

        $query = <<< QUERY
delete from relation
where ref_page = 0
QUERY;

        $sql = new Me_Sql();
        $sql->set_query($query);
        $sql->bind('where', 3, $page_id);
        $sql->exec();
        return true;
    }
}